# Funkcja do ładowania obrazów z podkatalogów w Pictures
def load_picture_from_subfolder(subfolder, filename)
  full_path = "#{subfolder}/#{filename}"
  return RPG::Cache.picture(full_path)
rescue
  raise "Błąd: nie udało się załadować obrazu: #{full_path}.png"
end

##########################################################
# The Arcane Container 1.0
# Haxo Game Studios                
# System odpowiedzialny za zarządzanie interaktywnymi skrzyniami w grze, 
# które mogą przechowywać przedmioty, broń, zbroje oraz wartości zmiennych.
# Skrypt pozwala na przypisywanie kolorów, dźwięków, opisów oraz grafik 
# do slotów w skrzyni, a także na wyświetlanie tła podczas interakcji z nią.
#
# Autor: Dawid Maxymilian Tomaszewski
# Adres mailowy: haxogamestudios@gmail.com
# Skrypt przeznaczony dla silnika RPG Maker XP i RPG Maker XPAce
#
# Licencja użytkowania skryptu "The Arcane Container 1.0"
# 1. Postanowienia ogólne
# Niniejsza licencja reguluje zasady korzystania ze skryptu The Arcane Container w wersji 1.0, 
# dalej zwanego "skryptem". Użytkownik, który korzysta ze skryptu, zgadza się na warunki niniejszej licencji.
#
# 2. Zezwolenia
#   - Korzystanie ze skryptu zarówno w grach komercyjnych, jak i niekomercyjnych.
#   - Modyfikację kodu w celu dostosowania go do specyfiki projektu, pod warunkiem zachowania warunków licencji.
#
# 3. Ograniczenia
#   - Zakazuje się publikacji skryptu w tej lub zmodyfikowanej wersji na innych witrynach niż
#     https://www.rpgmaker.pl/. Można uzyskać taką zgodę jedynie pisząc do mnie o udzielenie licencji 
#     zezwalającej na publikację w innym miejscu niż Twierdza.
#   - Zakazuje się sprzedaży skryptu w jakiejkolwiek formie, zarówno oryginalnej, jak i zmodyfikowanej.
#
# 4. Informacja o autorze
# Każdy projekt korzystający ze skryptu The Arcane Container 1.0 
# musi spełniać następujące warunki dotyczące uznania autorstwa:
#
# Podczas uruchamiania opcji "Nowa gra" należy wyświetlić poniższe informacje na ekranie przez
# minimum 7 sekund, pomijając czas przejścia efektów wizualnych, takich jak tzw. "smooth":
#
# [=][=][=][=][=][=][=][=][=][=][=][=][=][=][=][=][=][=][=][=][=]
#
# Nazwa: The Arcane Container 1.0
# Autor: Dawid Maxymilian Tomaszewski
# Studio: Haxo Game Studios
# Witryna: https://www.rpgmaker.pl/
# YouTube: HaxoGameStudiosOfficial
#
# [=][=][=][=][=][=][=][=][=][=][=][=][=][=][=][=][=][=][=][=][=]
#
# Wymagane jest również umieszczenie tej informacji w napisach końcowych gry.
#
# 5. Postanowienia końcowe
# Niniejsza licencja obowiązuje od momentu rozpoczęcia korzystania ze skryptu. 
# Użytkownik nie nabywa żadnych dodatkowych praw poza określonymi w tej licencji.
# Autor skryptu zastrzega sobie prawo do zmiany warunków licencji w przyszłych wersjach skryptu.
#
#
# Personalizacja polega na wklejeniu funkcji "Skrypt" i wrzuceniu poniższego kodu:
#
# Personalizacja w osobnym pliku TXT
##########################################################

class Game_System
  attr_accessor :chest_states

  alias initialize_with_chest_states initialize
  def initialize
    @chest_states = {}
    initialize_with_chest_states
  end
end

module ChestColors
  COLORS = {
    0 => Color.new(255, 255, 255),    # Bialy
    1 => Color.new(139, 0, 139),      # Fioletowy
    2 => Color.new(255, 0, 0),        # Czerwony
    3 => Color.new(0, 255, 0),        # Zielony
    4 => Color.new(30, 144, 255),     # Niebieski
    5 => Color.new(255, 96, 0),       # Pomaranczowy
    6 => Color.new(255, 255, 0),      # Zolty
    7 => Color.new(105, 105, 105),    # Szary
    8 => Color.new(0, 94, 128),       # Statystykowy
    9 => Color.new(72, 61, 139),      # Statystykowy-Fiolet
    10 => Color.new(166, 97, 74),     # Brązowy
    11 => Color.new(0, 102, 0),       # ZielonyZdrowy
    12 => Color.new(188, 143, 143),   # Ilosc Itemkow
    13 => Color.new(46, 139, 87),     # Statystykowa-Zielen
    14 => Color.new(123, 104, 238),   # Fiolet czekolady
    15 => Color.new(255, 101, 150),   # Statystykowy Niebieski
    16 => Color.new(107, 142, 35),    # Sklepikowy zielony
    17 => Color.new(165, 42, 42),     # Sklepikowy czerwony
    18 => Color.new(220, 20, 60),     # Czerwien medyczna
    19 => Color.new(70, 130, 180),    # Przedmioty questowe
    20 => Color.new(244, 164, 96),    # Miedziaki
    21 => Color.new(169, 169, 169),   # Srebrniki
    22 => Color.new(255, 215, 0),     # Denary
    23 => Color.new(173, 216, 230),   # Platyniaki
    24 => Color.new(0, 128, 128),     # Waluta elfow
    25 => Color.new(147, 112, 219),   # Elektryczne
    26 => Color.new(102, 205, 170),   # Nekromancja
    27 => Color.new(70, 130, 180),    # Zwoje Jasny Niebieski - szarawy
    28 => Color.new(221, 160, 221),   # Narzady
    29 => Color.new(255, 215, 0),     # Kolor zaru
    30 => Color.new(160, 82, 45),     # Kolor narzedzi
    31 => Color.new(60, 255, 178),    # Odpornosc: Powietrze
    32 => Color.new(25, 90, 255),     # Odpornosc: Woda
    33 => Color.new(164, 218, 82),    # Odpornosc: Gaz
    34 => Color.new(110, 220, 255)    # Odpornosc: Lod
  }

  def self.get_color(color_id)
    COLORS[color_id] || COLORS[0] # Domyślnie biały kolor, jeśli ID nie istnieje
  end
end

module ChestSounds
  SOUNDS = {
    0 => nil,  # Brak dźwięku
    1 => "TO-Pickupgold",  # Przykładowy dźwięk 1
    2 => "TO-PickupItem",  # Przykładowy dźwięk 2
    3 => "TO-PickupItemweapon"  # Przykładowy dźwięk 3
  }

  def self.get_sound(sound_id)
    SOUNDS[sound_id] # Zwraca nazwę dźwięku
  end
end

class ChestManager
  def self.set_chest_state(chest_id, used_slots, descriptions, colors, sounds)
    $game_system.chest_states[chest_id] = { "used_slots" => used_slots, "descriptions" => descriptions, "colors" => colors, "sounds" => sounds }
  end

  def self.get_chest_state(chest_id)
    $game_system.chest_states[chest_id] || { "used_slots" => Array.new(8, false), "descriptions" => Array.new(8, ""), "colors" => Array.new(8, 0), "sounds" => Array.new(8, 0) }
  end
end

class Scene_Chest
  def initialize(chest_id, slot_variables = [], slot_values = [], slot_graphics = [], background_image = nil, slot_descriptions = [], slot_colors = [], slot_sounds = [], container_image = "Container01")
    @chest_id = chest_id
    @background = Sprite.new

    if background_image
      @background.bitmap = RPG::Cache.picture(background_image)
    else
      @background.bitmap = RPG::Cache.picture("ZZZZZ-PANORAMA01")
    end

    if @background.bitmap
      @background.x = (640 - @background.bitmap.width) / 2
      @background.y = (480 - @background.bitmap.height) / 2
    else
      @background.x = 0
      @background.y = 0
      raise "Błąd: nie udało się załadować bitmapy tła"
    end

    @chest_graphic = Sprite.new
    @chest_graphic.bitmap = load_picture_from_subfolder("Chest", container_image) # Używamy dynamicznego obrazu kontenera

    if @chest_graphic.bitmap
      @chest_graphic.x = (640 - @chest_graphic.bitmap.width) / 2
      @chest_graphic.y = (480 - @chest_graphic.bitmap.height) / 2
    else
      @chest_graphic.x = 0
      @chest_graphic.y = 0
      raise "Błąd: nie udało się załadować bitmapy dla grafiki skrzyni"
    end

    @slots = []
    @slot_variables = slot_variables
    @slot_values = slot_values
    @slot_graphics = slot_graphics

    chest_state = ChestManager.get_chest_state(chest_id)
    @used_slots = chest_state["used_slots"]
    @saved_descriptions = chest_state["descriptions"]
    @saved_colors = chest_state["colors"]
    @saved_sounds = chest_state["sounds"]

    @slot_descriptions = []
    @saved_descriptions.each_with_index do |desc, i|
      @slot_descriptions[i] = desc.empty? ? slot_descriptions[i] : desc
    end

    @slot_colors = []
    @saved_colors.each_with_index do |color_id, i|
      @slot_colors[i] = slot_colors[i].nil? ? color_id : slot_colors[i]
    end

    @slot_sounds = []
    @saved_sounds.each_with_index do |sound_id, i|
      @slot_sounds[i] = slot_sounds[i].nil? ? sound_id : slot_sounds[i]
    end

    8.times do |i|
      slot = Sprite.new
      slot_graphic = @slot_graphics[i] || "Chest/ChestSlot"
      slot.bitmap = load_picture_from_subfolder("Chest", slot_graphic)

      if @used_slots[i]
        slot.bitmap = Bitmap.new(32, 32)
        @slot_descriptions[i] = ""
      end

      row = i / 4
      column = i % 4
      slot.x = @chest_graphic.x + 8 + column * 32
      slot.y = @chest_graphic.y + 39 + row * 32

      @slots << slot
    end

    @selected_index = 0
    @selection_indicator = Sprite.new
    @selection_indicator.bitmap = load_picture_from_subfolder("Chest", "ChestSlotS")
    update_slot_selection

    @description_text = Sprite.new
    @description_text.bitmap = Bitmap.new(640, 32)
    @description_text.x = 0
    @description_text.y = @chest_graphic.y + @chest_graphic.bitmap.height + 10
    update_description_text
  end

  def main
    Graphics.transition
    loop do
      Graphics.update
      Input.update
      update
      break if $scene != self
    end
    Graphics.freeze
    terminate
  end
  
  def update
    handle_input
  end

  def handle_input
    if Input.trigger?(Input::B) # Klawisz ESC
      $game_system.se_play($data_system.cancel_se)
      save_chest_state
      $scene = Scene_Map.new
      terminate
      return
    elsif Input.trigger?(Input::C) # Klawisz potwierdzenia
      confirm_selection
    elsif Input.repeat?(Input::LEFT)
      $game_system.se_play($data_system.cursor_se)
      move_selection(-1, true)
    elsif Input.repeat?(Input::RIGHT)
      $game_system.se_play($data_system.cursor_se)
      move_selection(1, true)
    elsif Input.repeat?(Input::UP)
      $game_system.se_play($data_system.cursor_se)
      move_selection(-4, false)
    elsif Input.repeat?(Input::DOWN)
      $game_system.se_play($data_system.cursor_se)
      move_selection(4, false)
    end
  end

  def move_selection(offset, horizontal = false)
    new_index = @selected_index + offset

    if horizontal
      if (new_index / 4) != (@selected_index / 4)
        return
      end
    end

    if new_index >= 0 && new_index < @slots.size
      @selected_index = new_index
      update_slot_selection
      update_description_text
    end
  end

  def confirm_selection
    unless @used_slots[@selected_index]
      variable_string = @slot_variables[@selected_index]
      value = @slot_values[@selected_index]
      sound_id = @slot_sounds[@selected_index]

      sound_name = ChestSounds.get_sound(sound_id)
      Audio.se_play("Audio/SE/#{sound_name}") if sound_name

      if variable_string
        if variable_string =~ /Item\[(\d+)\]/
          item_id = $1.to_i
          if $data_items[item_id]
            $game_party.gain_item(item_id, value)
            $game_system.se_play($data_system.decision_se)
          else
            p "Błąd: Nie znaleziono przedmiotu o ID #{item_id}"
          end
        elsif variable_string =~ /Weapon\[(\d+)\]/
          weapon_id = $1.to_i
          weapon = $data_weapons[weapon_id]
          if weapon
            $game_party.gain_weapon(weapon.id, value)
            $game_system.se_play($data_system.decision_se)
          else
            p "Błąd: Nie znaleziono broni o ID #{weapon_id}"
          end
        elsif variable_string =~ /Armor\[(\d+)\]/
          armor_id = $1.to_i
          armor = $data_armors[armor_id]
          if armor
            $game_party.gain_armor(armor.id, value)
            $game_system.se_play($data_system.decision_se)
          else
            p "Błąd: Nie znaleziono zbroi o ID #{armor_id}"
          end
        elsif variable_string =~ /Variable\[(\d+)\]/
          variable_id = $1.to_i
          $game_variables[variable_id] += value
          $game_system.se_play($data_system.decision_se)
        elsif variable_string == "Gold"
          $game_party.gain_gold(value)
          $game_system.se_play($data_system.decision_se)
        end
      end

      @used_slots[@selected_index] = true
      @slots[@selected_index].bitmap = Bitmap.new(32, 32)

      @slot_descriptions[@selected_index] = "" # Resetujemy opis
      @slot_colors[@selected_index] = 0

      save_chest_state

      update_description_text
    end
  end

  def update_slot_selection
    @slots.each_with_index do |slot, index|
      if @used_slots[index]
        slot.bitmap = Bitmap.new(32, 32)
      else
        slot_graphic = @slot_graphics[index] || "Chest/ChestSlot"
        slot.bitmap = load_picture_from_subfolder("Chest", slot_graphic)
      end
    end

    selected_slot = @slots[@selected_index]
    @selection_indicator.x = selected_slot.x
    @selection_indicator.y = selected_slot.y
    @selection_indicator.z = 1
  end

  def update_description_text
    @description_text.bitmap.clear
    base_description = @slot_descriptions[@selected_index] || " "
    slot_value = @slot_values[@selected_index]
    if @used_slots[@selected_index]
      description = ""  # Resetujemy opis dla użytych slotów
    else
      description = slot_value > 1 ? "#{base_description} x#{slot_value}" : base_description
    end
    color_id = @slot_colors[@selected_index] || 0
    color = ChestColors.get_color(color_id)
    @description_text.bitmap.font.color = color
    text_width = @description_text.bitmap.text_size(description).width
    x_position = (640 - text_width) / 2
    @description_text.bitmap.draw_text(x_position, 0, text_width, 32, description)
  end

  def save_chest_state
    ChestManager.set_chest_state(@chest_id, @used_slots, @slot_descriptions, @slot_colors, @slot_sounds)
  end

  def terminate
    @background.dispose if @background
    @chest_graphic.dispose if @chest_graphic
    @slots.each do |slot|
      slot.dispose
    end
    @selection_indicator.dispose if @selection_indicator
    @description_text.dispose if @description_text
  end
end
